@@ -37,6 +37,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "shadow.h"
 #include "blur.h"
 #include "contrast.h"
+#include "server_decoration.h"
 #include "slide.h"
 #include "shell.h"
 #include "shm_pool.h"
@@ -58,6 +59,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include <wayland-contrast-client-protocol.h>
 #include <wayland-slide-client-protocol.h>
 #include <wayland-dpms-client-protocol.h>
+#include <wayland-server-decoration-client-protocol.h>
 
 /*****
  * How to add another interface:
@@ -218,6 +220,13 @@ static const QMap<Registry::Interface, SuppertedInterfaceData> s_interfaces = {
         &org_kde_kwin_dpms_manager_interface,
         &Registry::dpmsAnnounced,
         &Registry::dpmsRemoved
+    }},
+    {Registry::Interface::ServerSideDecorationManager, {
+        1,
+        QByteArrayLiteral("org_kde_kwin_server_decoration_manager"),
+        &org_kde_kwin_server_decoration_manager_interface,
+        &Registry::serverSideDecorationManagerAnnounced,
+        &Registry::serverSideDecorationManagerRemoved
     }}
 };
 
@@ -496,6 +505,7 @@ BIND(Idle, org_kde_kwin_idle)
 BIND(FakeInput, org_kde_kwin_fake_input)
 BIND(OutputManagement, org_kde_kwin_outputmanagement)
 BIND(OutputDevice, org_kde_kwin_outputdevice)
+BIND(ServerSideDecorationManager, org_kde_kwin_server_decoration_manager)
 BIND2(ShadowManager, Shadow, org_kde_kwin_shadow_manager)
 BIND2(BlurManager, Blur, org_kde_kwin_blur_manager)
 BIND2(ContrastManager, Contrast, org_kde_kwin_contrast_manager)
@@ -547,6 +557,7 @@ CREATE(BlurManager)
 CREATE(ContrastManager)
 CREATE(SlideManager)
 CREATE(DpmsManager)
+CREATE(ServerSideDecorationManager)
 CREATE2(ShmPool, Shm)
 
 #undef CREATE
