@@ -21,9 +21,12 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "global_p.h"
 #include "resource_p.h"
 #include "display.h"
+#include "surface_interface.h"
 
 #include <QList>
 #include <QVector>
+#include <QRect>
+#include <QHash>
 
 #include <wayland-server.h>
 #include <wayland-plasma-window-management-server-protocol.h>
@@ -33,8 +36,6 @@ namespace KWayland
 namespace Server
 {
 
-static const quint32 s_version = 1;
-
 class PlasmaWindowManagementInterface::Private : public Global::Private
 {
 public:
@@ -56,6 +57,7 @@ private:
 
     PlasmaWindowManagementInterface *q;
     static const struct org_kde_plasma_window_management_interface s_interface;
+    static const quint32 s_version;
 };
 
 class PlasmaWindowInterface::Private
@@ -78,6 +80,7 @@ public:
     };
     QList<WindowResource> resources;
     quint32 windowId = 0;
+    QHash<SurfaceInterface*, QRect> minimizedGeometries;
 
 private:
     static void unbind(wl_resource *resource);
@@ -85,6 +88,8 @@ private:
     static void setStateCallback(wl_client *client, wl_resource *resource, uint32_t flags, uint32_t state);
     static void setVirtualDesktopCallback(wl_client *client, wl_resource *resource, uint32_t number);
     static void closeCallback(wl_client *client, wl_resource *resource);
+    static void setMinimizedGeometryCallback(wl_client *client, wl_resource *resource, wl_resource *panel, uint32_t x, uint32_t y, uint32_t width, uint32_t height);
+    static void unsetMinimizedGeometryCallback(wl_client *client, wl_resource *resource, wl_resource *panel);
     static Private *cast(wl_resource *resource) {
         return reinterpret_cast<Private*>(wl_resource_get_user_data(resource));
     }
@@ -100,16 +105,20 @@ private:
     static const struct org_kde_plasma_window_interface s_interface;
 };
 
+const quint32 PlasmaWindowManagementInterface::Private::s_version = 2;
+
 PlasmaWindowManagementInterface::Private::Private(PlasmaWindowManagementInterface *q, Display *d)
     : Global::Private(d, &org_kde_plasma_window_management_interface, s_version)
     , q(q)
 {
 }
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 const struct org_kde_plasma_window_management_interface PlasmaWindowManagementInterface::Private::s_interface = {
     showDesktopCallback,
     getWindowCallback
 };
+#endif
 
 void PlasmaWindowManagementInterface::Private::sendShowingDesktopState()
 {
@@ -234,11 +243,15 @@ PlasmaWindowInterface *PlasmaWindowManagementInterface::createWindow(QObject *pa
     return window;
 }
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 const struct org_kde_plasma_window_interface PlasmaWindowInterface::Private::s_interface = {
     setStateCallback,
     setVirtualDesktopCallback,
+    setMinimizedGeometryCallback,
+    unsetMinimizedGeometryCallback,
     closeCallback
 };
+#endif
 
 PlasmaWindowInterface::Private::Private(PlasmaWindowManagementInterface *wm, PlasmaWindowInterface *q)
     : q(q)
@@ -429,6 +442,48 @@ void PlasmaWindowInterface::Private::setStateCallback(wl_client *client, wl_reso
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREENABLE) {
         emit p->q->fullscreenableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREENABLE);
     }
+    if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPTASKBAR) {
+        emit p->q->skipTaskbarRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPTASKBAR);
+    }
+}
+
+void PlasmaWindowInterface::Private::setMinimizedGeometryCallback(wl_client *client, wl_resource *resource, wl_resource *panel, uint32_t x, uint32_t y, uint32_t width, uint32_t height)
+{
+    Q_UNUSED(client)
+    Private *p = cast(resource);
+    SurfaceInterface *panelSurface = SurfaceInterface::get(panel);
+
+    if (!panelSurface) {
+        return;
+    }
+
+    if (p->minimizedGeometries.value(panelSurface) == QRect(x, y, width, height)) {
+        return;
+    }
+
+    p->minimizedGeometries[panelSurface] = QRect(x, y, width, height);
+    emit p->q->minimizedGeometriesChanged();
+    connect(panelSurface, &QObject::destroyed, p->q, [p, panelSurface] () {
+        if (p->minimizedGeometries.remove(panelSurface)) {
+            emit p->q->minimizedGeometriesChanged();
+        }
+    });
+}
+
+void PlasmaWindowInterface::Private::unsetMinimizedGeometryCallback(wl_client *client, wl_resource *resource, wl_resource *panel)
+{
+    Q_UNUSED(client)
+    Private *p = cast(resource);
+    SurfaceInterface *panelSurface = SurfaceInterface::get(panel);
+
+    if (!panelSurface) {
+        return;
+    }
+    if (!p->minimizedGeometries.contains(panelSurface)) {
+        return;
+    }
+    p->minimizedGeometries.remove(panelSurface);
+    emit p->q->minimizedGeometriesChanged();
 }
 
 PlasmaWindowInterface::PlasmaWindowInterface(PlasmaWindowManagementInterface *wm, QObject *parent)
@@ -459,6 +514,11 @@ void PlasmaWindowInterface::unmap()
     d->unmap();
 }
 
+QHash<SurfaceInterface*, QRect>  PlasmaWindowInterface::minimizedGeometries() const
+{
+    return d->minimizedGeometries;
+}
+
 void PlasmaWindowInterface::setActive(bool set)
 {
     d->setState(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_ACTIVE, set);
@@ -519,6 +579,11 @@ void PlasmaWindowInterface::setMinimizeable(bool set)
     d->setState(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MINIMIZABLE, set);
 }
 
+void PlasmaWindowInterface::setSkipTaskbar(bool set)
+{
+    d->setState(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPTASKBAR, set);
+}
+
 void PlasmaWindowInterface::setThemedIconName(const QString &iconName)
 {
     d->setThemedIconName(iconName);
