@@ -81,9 +81,7 @@ void PointerInterface::Private::setCursor(quint32 serial, SurfaceInterface *surf
     if (!cursor) {
         Q_Q(PointerInterface);
         cursor = new Cursor(q);
-        cursor->d->enteredSerial = serial;
-        cursor->d->hotspot = hotspot;
-        cursor->d->surface = QPointer<SurfaceInterface>(surface);
+        cursor->d->update(QPointer<SurfaceInterface>(surface), serial, hotspot);
         QObject::connect(cursor, &Cursor::changed, q, &PointerInterface::cursorChanged);
         emit q->cursorChanged();
     } else {
@@ -101,10 +99,15 @@ const struct wl_pointer_interface PointerInterface::Private::s_interface = {
 PointerInterface::PointerInterface(SeatInterface *parent, wl_resource *parentResource)
     : Resource(new Private(parent, parentResource, this), parent)
 {
+    // TODO: handle touch
     connect(parent, &SeatInterface::pointerPosChanged, this, [this] {
         Q_D();
+        if (d->seat->isDragPointer()) {
+            // handled by DataDevice
+            return;
+        }
         if (d->focusedSurface && d->resource) {
-            const QPointF pos = d->seat->pointerPos() - d->seat->focusedPointerSurfacePosition();
+            const QPointF pos = d->seat->focusedPointerSurfaceTransformation().map(d->seat->pointerPos());
             wl_pointer_send_motion(d->resource, d->seat->timestamp(),
                                    wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
         }
@@ -134,7 +137,7 @@ void PointerInterface::setFocusedSurface(SurfaceInterface *surface, quint32 seri
         }
     );
 
-    const QPointF pos = d->seat->pointerPos() - d->seat->focusedPointerSurfacePosition();
+    const QPointF pos = d->seat->focusedPointerSurfaceTransformation().map(d->seat->pointerPos());
     wl_pointer_send_enter(d->resource, serial,
                           d->focusedSurface->resource(),
                           wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
