@@ -130,6 +130,25 @@ void SurfaceInterface::Private::setShadow(const QPointer<ShadowInterface> &shado
     pending.shadowIsSet = true;
 }
 
+void SurfaceInterface::Private::setBlur(const QPointer<BlurInterface> &blur)
+{
+    pending.blur = blur;
+    pending.blurIsSet = true;
+}
+
+void SurfaceInterface::Private::setSlide(const QPointer<SlideInterface> &slide)
+{
+    pending.slide = slide;
+    pending.slideIsSet = true;
+}
+
+void SurfaceInterface::Private::setContrast(const QPointer<ContrastInterface> &contrast)
+{
+    pending.contrast = contrast;
+    pending.contrastIsSet = true;
+}
+
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 const struct wl_surface_interface SurfaceInterface::Private::s_interface = {
     destroyCallback,
     attachCallback,
@@ -141,6 +160,7 @@ const struct wl_surface_interface SurfaceInterface::Private::s_interface = {
     bufferTransformCallback,
     bufferScaleCallback
 };
+#endif
 
 SurfaceInterface::SurfaceInterface(CompositorInterface *parent, wl_resource *parentResource)
     : Resource(new Private(this, parent, parentResource))
@@ -153,11 +173,15 @@ void SurfaceInterface::frameRendered(quint32 msec)
 {
     Q_D();
     // notify all callbacks
+    const bool needsFlush = !d->current.callbacks.isEmpty();
     while (!d->current.callbacks.isEmpty()) {
         wl_resource *r = d->current.callbacks.takeFirst();
         wl_callback_send_done(r, msec);
         wl_resource_destroy(r);
     }
+    if (needsFlush)  {
+        client()->flush();
+    }
 }
 
 void SurfaceInterface::Private::destroy()
@@ -186,6 +210,9 @@ void SurfaceInterface::Private::commit()
     const bool scaleFactorChanged = current.scale != pending.scale;
     const bool transformFactorChanged = current.transform != pending.transform;
     const bool shadowChanged = pending.shadowIsSet;
+    const bool blurChanged = pending.blurIsSet;
+    const bool contrastChanged = pending.contrastIsSet;
+    const bool slideChanged = pending.slideIsSet;
     bool sizeChanged = false;
     auto buffer = current.buffer;
     if (bufferChanged) {
@@ -207,6 +234,18 @@ void SurfaceInterface::Private::commit()
     if (shadowChanged) {
         shadow = pending.shadow;
     }
+    auto blur = current.blur;
+    if (blurChanged) {
+        blur = pending.blur;
+    }
+    auto contrast = current.contrast;
+    if (contrastChanged) {
+        contrast = pending.contrast;
+    }
+    auto slide = current.slide;
+    if (slideChanged) {
+        slide = pending.slide;
+    }
     QList<wl_resource*> callbacks = current.callbacks;
     callbacks.append(pending.callbacks);
     // copy values
@@ -214,6 +253,9 @@ void SurfaceInterface::Private::commit()
     current.buffer = buffer;
     current.callbacks = callbacks;
     current.shadow = shadow;
+    current.blur = blur;
+    current.contrast = contrast;
+    current.slide = slide;
     pending = State{};
     pending.children = current.children;
     pending.input = current.input;
@@ -255,6 +297,15 @@ void SurfaceInterface::Private::commit()
     if (shadowChanged) {
         emit q->shadowChanged();
     }
+    if (blurChanged) {
+        emit q->blurChanged();
+    }
+    if (contrastChanged) {
+        emit q->contrastChanged();
+    }
+    if (slideChanged) {
+        emit q->slideOnShowHideChanged();
+    }
 }
 
 void SurfaceInterface::Private::damage(const QRect &rect)
@@ -413,6 +464,11 @@ QRegion SurfaceInterface::input() const
 }
 
 bool SurfaceInterface::inputIsInfitine() const
+{
+    return inputIsInfinite();
+}
+
+bool SurfaceInterface::inputIsInfinite() const
 {
     Q_D();
     return d->current.inputIsInfinite;
@@ -480,6 +536,24 @@ QPointer< ShadowInterface > SurfaceInterface::shadow() const
     return d->current.shadow;
 }
 
+QPointer< BlurInterface > SurfaceInterface::blur() const
+{
+    Q_D();
+    return d->current.blur;
+}
+
+QPointer< ContrastInterface > SurfaceInterface::contrast() const
+{
+    Q_D();
+    return d->current.contrast;
+}
+
+QPointer< SlideInterface > SurfaceInterface::slideOnShowHide() const
+{
+    Q_D();
+    return d->current.slide;
+}
+
 SurfaceInterface::Private *SurfaceInterface::d_func() const
 {
     return reinterpret_cast<Private*>(d.data());
